#ifndef AUBO_SCOPE_KEYBOARD_HELPER_H
#define AUBO_SCOPE_KEYBOARD_HELPER_H

#include <functional>

#include <QApplication>
#include <QWidget>
#include <QLineEdit>
#include <QPlainTextEdit>
#include <QComboBox>
#include <QMap>

#include <aubo_caps/domain/user_interface/keyboard_input_factory.h>
#include <aubo_caps/domain/value/expression/expression.h>
#include <aubo_caps/contribution/view_api_provider.h>

namespace arcs {
namespace aubo_scope {

/**
 * 自动处理 QLineEdit 和 QPlainTextEdit 的键盘弹窗
 *
 * ps. KeyboardHelper会自动注册所有的 QLineEdit(数字键盘) 和
 * QPlainTextEdit(标准键盘) 控件
 */
class Q_DECL_EXPORT KeyboardHelper : public QObject
{
    Q_OBJECT

public:
    /// 键盘类型定义
    enum Type
    {
        StandardKeyboard, ///< 标准键盘,输入字符没有限制
        StringKeyboard, ///< 用于命名的字符串键盘,无法输入特殊字符
        PasswordKeyboard, ///<密码键盘,用于输入特定格式的密码
        NumberKeyboard,   ///< 标准数字键盘,输入数字类型没有限制
        DoubleKeypad,     ///<浮点数数字键盘
        PositiveDoubleKeypad,  ///<正浮点数数字键盘
        IntegerKeypad,         ///<整数数字键盘
        PositiveIntegerKeypad, ///<正整数数字键盘
        IPAddressKeyboard,     ///<用于输入IP的键盘
    };

    explicit KeyboardHelper(QObject *parent = NULL);

    ~KeyboardHelper();

    void setFactory(const std::shared_ptr<KeyboardInputFactory> &factory);

    bool showKeyboard(std::shared_ptr<KeyboardTextInput> kb, QLineEdit *le);

    bool showKeyboard(std::shared_ptr<KeyboardDoubleInput> kb, QLineEdit *le);
    bool showKeyboard(std::shared_ptr<KeyboardIntInput> kb, QLineEdit *le);
    bool showKeyboard(std::shared_ptr<KeyboardUIntInput> kb, QLineEdit *le);

    /**
     * 注册QLineEdit控件键盘
     *
     * @param le 控件指针
     * @param type 键盘类型
     */
    void installLineEdit(QLineEdit *le, int type);

    void installLineEdit(QLineEdit *le, int type, std::function<void()> &&func);

    void installLineEdit(QPlainTextEdit *le, int type);
    void uninstallLineEdit(QLineEdit *le);
    void uninstallLineEdit(QPlainTextEdit *pe);

    /// 将所有的 QLineEdit QPlainTextEdit 安装时间过滤器 installEventFilter
    void installQLineEdits(QWidget *wd, Qt::FindChildOptions options =
                                            Qt::FindChildrenRecursively);

    bool eventFilter(QObject *obj, QEvent *event) override;

private:
    bool processLineEdits(QObject *obj, QEvent *event);
    bool processPlainEdits(QObject *obj, QEvent *event);

private:
    std::shared_ptr<KeyboardInputFactory> factory_{ nullptr };

    QHash<QWidget *, std::function<void()>> slots_map_;
};

} // namespace aubo_scope
} // namespace arcs

#endif
